@@ -85,7 +85,7 @@ const Clock = new Lang.Class({
         let date = new Date();
         /* Translators: This is a time format for a date in
            long format */
-        let dateFormat = Shell.util_translate_time_string("%A, %B %d");
+        let dateFormat = Shell.util_translate_time_string(N_("%A, %B %d"));
         this._date.text = date.toLocaleFormat(dateFormat);
     },
 
@@ -670,7 +670,9 @@ const ScreenShield = new Lang.Class({
         if (this._lockScreenState != MessageTray.State.SHOWN)
             return Clutter.EVENT_PROPAGATE;
 
-        let isEnter = (symbol == Clutter.KEY_Return || symbol == Clutter.KEY_KP_Enter);
+        let isEnter = (symbol == Clutter.KEY_Return ||
+                       symbol == Clutter.KEY_KP_Enter ||
+                       symbol == Clutter.KEY_ISO_Enter);
         if (!isEnter && !(GLib.unichar_isprint(unichar) || symbol == Clutter.KEY_Escape))
             return Clutter.EVENT_PROPAGATE;
 
@@ -1333,6 +1335,8 @@ const ScreenShield = new Lang.Class({
 
     // If the previous shell crashed, and gnome-session restarted us, then re-lock
     lockIfWasLocked: function() {
+        if (!this._settings.get_boolean(LOCK_ENABLED_KEY))
+            return;
         let wasLocked = global.get_runtime_state('b', LOCKED_STATE_STR);
         if (wasLocked === null)
             return;
